@@ -3,16 +3,19 @@ import os
 import shutil
 import threading
 import time
-from difflib import get_close_matches
 from logging import getLogger, StreamHandler, Formatter
 from types import SimpleNamespace
 from zipfile import BadZipFile
 
 import numpy as np
 from PIL import Image
+from flask import Flask
+from flask_sqlalchemy import SQLAlchemy
+from sqlalchemy import event
 from sqlalchemy.inspection import inspect
 from sqlalchemy.sql.expression import case
-from sqlalchemy import event
+
+__version__ = "0.0.6"
 
 Image.LOAD_TRUNCATED_IMAGES = True  # this stops truncated image errors on large image files
 
@@ -26,8 +29,7 @@ logger.addHandler(handler)
 
 class MultiFileSaver(object):
     """
-    This class is used as a cool down for saving the new image data
-    It also backs up the most recent version of indexed data before save so that if the program is terminated it dose not leave a corrupted file
+    Safely saves file using a cool-down timer and makes a backup of a file when it is saved
     """
 
     def __init__(self):
@@ -37,6 +39,7 @@ class MultiFileSaver(object):
         self.time_timer = None
 
     def save(self):
+        """writes the the stored file data to all the files """
         # fist clear all the timers
         if self.threading_timer is not None:
             self.threading_timer.cancel()
@@ -53,9 +56,16 @@ class MultiFileSaver(object):
         self.wait_time = total_time + .5
 
     def saver(self, file_path):
-        """this function returns inner saver that can be called to update the save data"""
+        """this function returns inner saver that can be called to update the save data
+
+        :param file_path: path that the data is saved to
+        """
 
         def inner_saver(new_data):
+            """update the file data and updates cool-down timers
+
+            :param new_data: the new file data
+            """
             self.save_data[file_path] = new_data
             if self.threading_timer is not None:  # check there is a timer set
                 self.threading_timer.cancel()
@@ -77,6 +87,32 @@ class MultiFileSaver(object):
 
 
 class ImageSearch(object):
+    """This is used to add image searching functionality to your flask app
+
+    there are two ways to setup flask image search.
+
+    .. code-block:: python
+
+        app = Flask(__name__)
+        db = SQLAlchemy(app)
+        image_search = ImageSearch(app, db)
+
+    or
+
+    .. code-block:: python
+
+        db = SQLAlchemy()
+        image_search = ImageSearch()
+
+        def create_app():
+            app = Flask(__name__)
+            db.init_app(app)
+            image_search.init_app(app, db)
+            return app
+
+
+    """
+
     def __init__(self, app, db, no_tf=None):
         self.app = app
         self.models = {}
@@ -85,6 +121,17 @@ class ImageSearch(object):
             self.app_init(app, db, no_tf)
 
     def app_init(self, app, db, no_tf=None):
+        """If a app is given this function calls app_init and passes on the values.
+
+        :param app: the project's flask app
+        :type app: Flask
+        :param db: the project's SQLAlchemy instance
+        :type db: SQLAlchemy
+        :param no_tf: If this is set to None app.debug will be used. if this is true tensorflow is not loaded and the
+            debug version of the feature extractor will. if the debug version is used it will use random features
+        :type no_tf: bool or None
+
+        """
         if no_tf is None:
             if app.debug:
                 """tensorflow has errors running in debug with reload"""
@@ -117,11 +164,14 @@ class ImageSearch(object):
 
     def register(self, url_column='url', id_col=None, fk_cols=None):
         """
-        decorator for registering models to be indexed
+        decorator for registering models to be indexed. adds event listener for model changes
+
         :param url_column: the column where the url is located
+        :type url_column: str
         :param id_col: the name of the id column to be used
+        :type id_col: str
         :param fk_cols: list of the names foreign keys to be tracked
-        :return:
+        :type url_column: list[int]
         """
         if fk_cols is None:
             fk_cols = []
@@ -195,7 +245,12 @@ class ImageSearch(object):
         return decorator_inner
 
     def index(self, entry, override=False):
-        """this is for indexing one image"""
+        """this is for indexing one image
+
+        :param entry: model to be indexed
+        :param override: If the image is already indexed should it be re-indexed
+        :type override: bool
+        """
         self.is_running.wait()
         self.is_registering.wait()
 
@@ -215,7 +270,12 @@ class ImageSearch(object):
         return True
 
     def index_model(self, model, override=False):
-        """index entire model"""
+        """index entire model
+
+        :param model: this is the model that will be queried to get all the images
+        :param override: If the image is already indexed should it be re-indexed
+        :type override: bool
+        """
 
         def thread_content():  # run the function in the backgound
             self.is_registering.wait()
@@ -245,7 +305,10 @@ class ImageSearch(object):
         thread.start()
 
     def delete_index(self, model):
-        """delete an index"""
+        """delete an index
+
+        :param model: the model with the image to be deleted
+        """
         data = self.models[model.__tablename__]
         # work out the feature id of the indexed data
         feature_id = "_".join([str(getattr(model, id_column)) for id_column in [data.id_column] + data.fk_columns])
@@ -257,6 +320,21 @@ class ImageSearch(object):
             pass
 
     def search(self, model_name, image_path=None, image_data=None, limit=-1):
+        """this is used to search the images, it returns a list of tuples that contain the feature_if of a image and the
+         distance from the query image
+
+        :param model_name: this is the name of the model that is connected to the indexed data
+        :type model_name: str
+        :param image_path: this is the path to the query image. if this is not set image_data must be
+        :type image_path: str or None
+        :param image_data: the query as a PIL Image object
+        :type image_data: Image
+        :param limit: this is the amount of results to return. if this is set to -1 all the results will be returned
+        :type limit: int
+
+        :returns: this returns a list of feature_ids and the distances from the search image
+        :rtype: list[tuple[str, float]]
+        """
         self.is_running.wait()  # wait till the app is running
         self.is_registering.wait()  # wait till the app is not registering a model_
         if image_path is not None:  # if the image path is being used set image_data to the opened image
@@ -276,6 +354,24 @@ class ImageSearch(object):
         return [(feature_ids[uid], distances[uid]) for uid in distances_sorted]
 
     def query_search(self, model=None, image_path=None, image_data=None, limit=20):
+        """This function is used to search for an image in a model
+
+        .. code-block:: python
+
+            db.session.query(Image).with_transformation(image_search.query_search(image_path='query.jpg'))
+
+        :param model: this is the model that has the indexed images. if this is None the model will be worked out form
+            the query. the model must be set if the query has more than one model or an error will be thrown
+        :param image_path: this is the path to the query image. if this is not set image_data must be
+        :type image_path: str or None
+        :param image_data: the query image as a PIL Image object
+        :type image_data: Image
+        :param limit: this is the amount of results to return
+        :type limit: int
+
+        :returns: this returns a filtered query
+        """
+
         def inner(query):
             if image_data is None and image_path is None:
                 return query
@@ -309,9 +405,27 @@ class ImageSearch(object):
         return inner
 
     def query_relation_search(self, model=None, model_id_column=None, relation_column=None, foreign_key_column=None,
-                              image_path=None,
-                              image_data=None, limit=20):
-        # todo: make this call query search to cut down on duplicate code
+                              image_path=None, image_data=None, limit=10):
+        """This is like query_search but it works on non indexed models that have a relationship to an indexed model.
+
+        :param model: the model that being query. if this is None the model will be worked out form
+            the query. the model must be set if the query has more than one model or an error will be thrown
+        :param model_id_column: this is the id of the model being queried. if this is None this function will try to
+            find a valid id if no valid id can be found then an error will be thrown and you will need to set it.
+        :param relation_column: this is the relationship column that connects the model and the model that
+            has indexed images, if this is None this function will try and work out the correct column if no column is
+            found an error will be thrown and you will need to set this
+        :param foreign_key_column: this is the foreign key column that connects the model with the indexed images
+            and the one being queried
+        :param image_path: this is the path to the query image. if this is not set image_data must be
+        :type image_path: str or None
+        :param image_data: the query image as a PIL Image object
+        :type image_data: Image
+        :param limit: this is the amount of results to return. this doesn't limit the number of images
+        :type limit: int
+
+        :returns: this returns a filtered query that is joined with the related model
+        """
         if image_path is not None or image_data is not None:
             def inner(query):
                 self.is_running.wait()
